@@ -1,19 +1,44 @@
+from __future__ import annotations
+
 import datetime
 import json
 import random
-from typing import List, Optional
+from typing import List, Optional, Union
 
-from six import string_types
+from mcstatus.address import Address
+from mcstatus.protocol.connection import Connection, TCPAsyncSocketConnection, TCPSocketConnection
 
-from mcstatus.protocol.connection import Connection
+STYLE_MAP = {
+    "bold": "l",
+    "italic": "o",
+    "underlined": "n",
+    "obfuscated": "k",
+    "color": {
+        "dark_red": "4",
+        "red": "c",
+        "gold": "6",
+        "yellow": "e",
+        "dark_green": "2",
+        "green": "a",
+        "aqua": "b",
+        "dark_aqua": "3",
+        "dark_blue": "1",
+        "blue": "9",
+        "light_purple": "d",
+        "dark_purple": "5",
+        "white": "f",
+        "gray": "7",
+        "dark_gray": "8",
+        "black": "0",
+    },
+}
 
 
 class ServerPinger:
     def __init__(
         self,
-        connection,
-        host: str = "",
-        port: int = 0,
+        connection: TCPSocketConnection,
+        address: Address,
         version: int = 47,
         ping_token=None,
     ):
@@ -21,21 +46,20 @@ class ServerPinger:
             ping_token = random.randint(0, (1 << 63) - 1)
         self.version = version
         self.connection = connection
-        self.host = host
-        self.port = port
+        self.address = address
         self.ping_token = ping_token
 
-    def handshake(self):
+    def handshake(self) -> None:
         packet = Connection()
         packet.write_varint(0)
         packet.write_varint(self.version)
-        packet.write_utf(self.host)
-        packet.write_ushort(self.port)
+        packet.write_utf(self.address.host)
+        packet.write_ushort(self.address.port)
         packet.write_varint(1)  # Intention to query status
 
         self.connection.write_buffer(packet)
 
-    def read_status(self):
+    def read_status(self) -> "PingResponse":
         request = Connection()
         request.write_varint(0)  # Request status
         self.connection.write_buffer(request)
@@ -50,9 +74,9 @@ class ServerPinger:
         try:
             return PingResponse(raw)
         except ValueError as e:
-            raise IOError("Received invalid status response: %s" % e)
+            raise IOError(f"Received invalid status response: {e}")
 
-    def test_ping(self):
+    def test_ping(self) -> float:
         request = Connection()
         request.write_varint(1)  # Test ping
         request.write_long(self.ping_token)
@@ -66,16 +90,26 @@ class ServerPinger:
         received_token = response.read_long()
         if received_token != self.ping_token:
             raise IOError(
-                "Received mangled ping response packet (expected token %d, received %d)" % (self.ping_token, received_token)
+                f"Received mangled ping response packet (expected token {self.ping_token}, received {received_token})"
             )
 
         delta = received - sent
-        # We have no trivial way of getting a time delta :(
-        return (delta.days * 24 * 60 * 60 + delta.seconds) * 1000 + delta.microseconds / 1000.0
+        return delta.total_seconds() * 1000
 
 
 class AsyncServerPinger(ServerPinger):
-    async def read_status(self):
+    def __init__(
+        self,
+        connection: TCPAsyncSocketConnection,
+        address: Address,
+        version: int = 47,
+        ping_token=None,
+    ):
+        # We do this to inform python about self.connection type (it's async)
+        super().__init__(connection, address=address, version=version, ping_token=ping_token)  # type: ignore[arg-type]
+        self.connection: TCPAsyncSocketConnection
+
+    async def read_status(self) -> "PingResponse":
         request = Connection()
         request.write_varint(0)  # Request status
         self.connection.write_buffer(request)
@@ -90,9 +124,9 @@ class AsyncServerPinger(ServerPinger):
         try:
             return PingResponse(raw)
         except ValueError as e:
-            raise IOError("Received invalid status response: %s" % e)
+            raise IOError(f"Received invalid status response: {e}")
 
-    async def test_ping(self):
+    async def test_ping(self) -> float:
         request = Connection()
         request.write_varint(1)  # Test ping
         request.write_long(self.ping_token)
@@ -106,12 +140,11 @@ class AsyncServerPinger(ServerPinger):
         received_token = response.read_long()
         if received_token != self.ping_token:
             raise IOError(
-                "Received mangled ping response packet (expected token %d, received %d)" % (self.ping_token, received_token)
+                f"Received mangled ping response packet (expected token {self.ping_token}, received {received_token})"
             )
 
         delta = received - sent
-        # We have no trivial way of getting a time delta :(
-        return (delta.days * 24 * 60 * 60 + delta.seconds) * 1000 + delta.microseconds / 1000.0
+        return delta.total_seconds() * 1000
 
 
 class PingResponse:
@@ -124,18 +157,18 @@ class PingResponse:
 
             def __init__(self, raw):
                 if not isinstance(raw, dict):
-                    raise ValueError("Invalid player object (expected dict, found %s" % type(raw))
+                    raise ValueError(f"Invalid player object (expected dict, found {type(raw)}")
 
                 if "name" not in raw:
                     raise ValueError("Invalid player object (no 'name' value)")
-                if not isinstance(raw["name"], string_types):
-                    raise ValueError("Invalid player object (expected 'name' to be str, was %s)" % type(raw["name"]))
+                if not isinstance(raw["name"], str):
+                    raise ValueError(f"Invalid player object (expected 'name' to be str, was {type(raw['name'])}")
                 self.name = raw["name"]
 
                 if "id" not in raw:
                     raise ValueError("Invalid player object (no 'id' value)")
-                if not isinstance(raw["id"], string_types):
-                    raise ValueError("Invalid player object (expected 'id' to be str, was %s)" % type(raw["id"]))
+                if not isinstance(raw["id"], str):
+                    raise ValueError(f"Invalid player object (expected 'id' to be str, was {type(raw['id'])}")
                 self.id = raw["id"]
 
         online: int
@@ -144,23 +177,23 @@ class PingResponse:
 
         def __init__(self, raw):
             if not isinstance(raw, dict):
-                raise ValueError("Invalid players object (expected dict, found %s" % type(raw))
+                raise ValueError(f"Invalid players object (expected dict, found {type(raw)}")
 
             if "online" not in raw:
                 raise ValueError("Invalid players object (no 'online' value)")
             if not isinstance(raw["online"], int):
-                raise ValueError("Invalid players object (expected 'online' to be int, was %s)" % type(raw["online"]))
+                raise ValueError(f"Invalid players object (expected 'online' to be int, was {type(raw['online'])})")
             self.online = raw["online"]
 
             if "max" not in raw:
                 raise ValueError("Invalid players object (no 'max' value)")
             if not isinstance(raw["max"], int):
-                raise ValueError("Invalid players object (expected 'max' to be int, was %s)" % type(raw["max"]))
+                raise ValueError(f"Invalid players object (expected 'max' to be int, was {type(raw['max'])}")
             self.max = raw["max"]
 
             if "sample" in raw:
                 if not isinstance(raw["sample"], list):
-                    raise ValueError("Invalid players object (expected 'sample' to be list, was %s)" % type(raw["max"]))
+                    raise ValueError(f"Invalid players object (expected 'sample' to be list, was {type(raw['max'])})")
                 self.sample = [PingResponse.Players.Player(p) for p in raw["sample"]]
             else:
                 self.sample = None
@@ -171,18 +204,18 @@ class PingResponse:
 
         def __init__(self, raw):
             if not isinstance(raw, dict):
-                raise ValueError("Invalid version object (expected dict, found %s" % type(raw))
+                raise ValueError(f"Invalid version object (expected dict, found {type(raw)})")
 
             if "name" not in raw:
                 raise ValueError("Invalid version object (no 'name' value)")
-            if not isinstance(raw["name"], string_types):
-                raise ValueError("Invalid version object (expected 'name' to be str, was %s)" % type(raw["name"]))
+            if not isinstance(raw["name"], str):
+                raise ValueError(f"Invalid version object (expected 'name' to be str, was {type(raw['name'])})")
             self.name = raw["name"]
 
             if "protocol" not in raw:
                 raise ValueError("Invalid version object (no 'protocol' value)")
             if not isinstance(raw["protocol"], int):
-                raise ValueError("Invalid version object (expected 'protocol' to be int, was %s)" % type(raw["protocol"]))
+                raise ValueError(f"Invalid version object (expected 'protocol' to be int, was {type(raw['protocol'])})")
             self.protocol = raw["protocol"]
 
     players: Players
@@ -204,12 +237,34 @@ class PingResponse:
 
         if "description" not in raw:
             raise ValueError("Invalid status object (no 'description' value)")
-        if isinstance(raw["description"], dict):
-            self.description = raw["description"]["text"]
-        else:
-            self.description = raw["description"]
+        self.description = self._parse_description(raw["description"])
 
-        if "favicon" in raw:
-            self.favicon = raw["favicon"]
+        self.favicon = raw.get("favicon")
+
+    @staticmethod
+    def _parse_description(raw_description: Union[dict, list, str]) -> str:
+        if isinstance(raw_description, str):
+            return raw_description
+
+        if isinstance(raw_description, dict):
+            entries = raw_description.get("extra", [])
+            end = raw_description["text"]
         else:
-            self.favicon = None
+            entries = raw_description
+            end = ""
+
+        description = ""
+
+        for entry in entries:
+            for style_key, style_val in STYLE_MAP.items():
+                if entry.get(style_key):
+                    try:
+                        if isinstance(style_val, dict):
+                            style_val = style_val[entry[style_key]]
+
+                        description += f"§{style_val}"
+                    except KeyError:
+                        pass  # ignoring these key errors strips out html color codes
+            description += entry.get("text", "")
+
+        return description + end
