@@ -1,8 +1,14 @@
-import random
-import struct
-from typing import List
+from __future__ import annotations
 
-from mcstatus.protocol.connection import Connection
+import random
+import re
+import struct
+from typing import List, TYPE_CHECKING
+
+from mcstatus.protocol.connection import Connection, UDPAsyncSocketConnection, UDPSocketConnection
+
+if TYPE_CHECKING:
+    from typing_extensions import Self
 
 
 class ServerQuerier:
@@ -11,16 +17,16 @@ class ServerQuerier:
     PACKET_TYPE_CHALLENGE = 9
     PACKET_TYPE_QUERY = 0
 
-    def __init__(self, connection):
+    def __init__(self, connection: UDPSocketConnection):
         self.connection = connection
         self.challenge = 0
 
     @staticmethod
-    def _generate_session_id():
+    def _generate_session_id() -> int:
         # minecraft only supports lower 4 bits
-        return random.randint(0, 2 ** 31) & 0x0F0F0F0F
+        return random.randint(0, 2**31) & 0x0F0F0F0F
 
-    def _create_packet(self):
+    def _create_packet(self) -> Connection:
         packet = Connection()
         packet.write(self.MAGIC_PREFIX)
         packet.write(struct.pack("!B", self.PACKET_TYPE_QUERY))
@@ -29,26 +35,26 @@ class ServerQuerier:
         packet.write(self.PADDING)
         return packet
 
-    def _create_handshake_packet(self):
+    def _create_handshake_packet(self) -> Connection:
         packet = Connection()
         packet.write(self.MAGIC_PREFIX)
         packet.write(struct.pack("!B", self.PACKET_TYPE_CHALLENGE))
         packet.write_uint(self._generate_session_id())
         return packet
 
-    def _read_packet(self):
+    def _read_packet(self) -> Connection:
         packet = Connection()
         packet.receive(self.connection.read(self.connection.remaining()))
         packet.read(1 + 4)
         return packet
 
-    def handshake(self):
+    def handshake(self) -> None:
         self.connection.write(self._create_handshake_packet())
 
         packet = self._read_packet()
         self.challenge = int(packet.read_ascii())
 
-    def read_query(self):
+    def read_query(self) -> QueryResponse:
         request = self._create_packet()
         self.connection.write(request)
 
@@ -57,19 +63,24 @@ class ServerQuerier:
 
 
 class AsyncServerQuerier(ServerQuerier):
-    async def _read_packet(self):
+    def __init__(self, connection: UDPAsyncSocketConnection):
+        # We do this to inform python about self.connection type (it's async)
+        super().__init__(connection)  # type: ignore[arg-type]
+        self.connection: UDPAsyncSocketConnection
+
+    async def _read_packet(self) -> Connection:
         packet = Connection()
         packet.receive(await self.connection.read(self.connection.remaining()))
         packet.read(1 + 4)
         return packet
 
-    async def handshake(self):
+    async def handshake(self) -> None:
         await self.connection.write(self._create_handshake_packet())
 
         packet = await self._read_packet()
         self.challenge = int(packet.read_ascii())
 
-    async def read_query(self):
+    async def read_query(self) -> QueryResponse:
         request = self._create_packet()
         await self.connection.write(request)
 
@@ -119,29 +130,37 @@ class QueryResponse:
             self.map = raw["map"]
             self.players = QueryResponse.Players(raw["numplayers"], raw["maxplayers"], players)
             self.software = QueryResponse.Software(raw["version"], raw["plugins"])
-        except:
+        except KeyError:
             raise ValueError("The provided data is not valid")
 
     @classmethod
-    def from_connection(cls, response: Connection):
+    def from_connection(cls, response: Connection) -> Self:
         response.read(len("splitnum") + 1 + 1 + 1)
         data = {}
         players = []
 
         while True:
             key = response.read_ascii()
-            if len(key) == 0:
+            if key == "hostname":  # hostname is actually motd in the query protocol
+                match = re.search(b"(.*)\x00gametype", response.received, flags=re.DOTALL)
+                motd = match.group(1) if match else ""
+                # Since the query protocol does not properly support unicode, the motd is still not resolved
+                # correctly; however, this will avoid other parameter parsing errors.
+                data[key] = response.read(len(motd)).decode("ISO-8859-1")
+                response.read(1)  # ignore null byte
+            elif len(key) == 0:
                 response.read(1)
                 break
-            value = response.read_ascii()
-            data[key] = value
+            else:
+                value = response.read_ascii()
+                data[key] = value
 
         response.read(len("player_") + 1 + 1)
 
         while True:
-            name = response.read_ascii()
-            if len(name) == 0:
+            motd = response.read_ascii()
+            if len(motd) == 0:
                 break
-            players.append(name)
+            players.append(motd)
 
         return cls(data, players)
