@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 import socket
-from typing import Any
-import click
 from json import dumps as json_dumps
 
-from .. import MinecraftServer
+import click
 
-server = None
+from mcstatus import MinecraftServer
+
+server: MinecraftServer = None  # type: ignore[assignment]  # This will be set with cli function
 
 
 @click.group(context_settings=dict(help_option_names=["-h", "--help"]))
@@ -49,7 +51,7 @@ def ping():
     """
     Ping server for latency.
     """
-    click.echo("{}ms".format(server.ping()))
+    click.echo(f"{server.ping()}ms")
 
 
 @cli.command(short_help="basic server information")
@@ -59,17 +61,14 @@ def status():
     servers that are version 1.7 or higher.
     """
     response = server.status()
-    click.echo("version: v{} (protocol {})".format(response.version.name, response.version.protocol))
-    click.echo('description: "{}"'.format(response.description))
-    click.echo(
-        "players: {}/{} {}".format(
-            response.players.online,
-            response.players.max,
-            ["{} ({})".format(player.name, player.id) for player in response.players.sample]
-            if response.players.sample is not None
-            else "No players online",
-        )
-    )
+    if response.players.sample is not None:
+        player_sample = str([f"{player.name} ({player.id})" for player in response.players.sample])
+    else:
+        player_sample = "No players online"
+
+    click.echo(f"version: v{response.version.name} (protocol {response.version.protocol})")
+    click.echo(f'description: "{response.description}"')
+    click.echo(f"players: {response.players.online}/{response.players.max} {player_sample}")
 
 
 @cli.command(short_help="all available server information in json")
@@ -96,12 +95,12 @@ def json():
         if status_res.players.sample is not None:
             data["players"] = [{"name": player.name, "id": player.id} for player in status_res.players.sample]
 
-        query_res = server.query(tries=1)
+        query_res = server.query(tries=1)  # type: ignore[call-arg] # tries is supported with retry decorator
         data["host_ip"] = query_res.raw["hostip"]
         data["host_port"] = query_res.raw["hostport"]
         data["map"] = query_res.map
         data["plugins"] = query_res.software.plugins
-    except:
+    except Exception:  # TODO: Check what this actually excepts
         pass
     click.echo(json_dumps(data))
 
@@ -116,23 +115,18 @@ def query():
         response = server.query()
     except socket.timeout:
         print(
-            """The server did not respond to the query protocol.
-              Please ensure that the server has enable-query turned on, and that the necessary port (same as server-port unless query-port is set) is open in any firewall(s).
-              See https://wiki.vg/Query for further information."""
+            "The server did not respond to the query protocol."
+            "\nPlease ensure that the server has enable-query turned on,"
+            " and that the necessary port (same as server-port unless query-port is set) is open in any firewall(s)."
+            "\nSee https://wiki.vg/Query for further information."
         )
         raise click.Abort()
-    click.echo("host: {}:{}".format(response.raw["hostip"], response.raw["hostport"]))
-    click.echo("software: v{} {}".format(response.software.version, response.software.brand))
-    click.echo("plugins: {}".format(response.software.plugins))
-    click.echo('motd: "{}"'.format(response.motd))
-    click.echo(
-        "players: {}/{} {}".format(
-            response.players.online,
-            response.players.max,
-            response.players.names,
-        )
-    )
+    click.echo(f"host: {response.raw['hostip']}:{response.raw['hostport']}")
+    click.echo(f"software: v{response.software.version} {response.software.brand}")
+    click.echo(f"plugins: {response.software.plugins}")
+    click.echo(f'motd: "{response.motd}"')
+    click.echo(f"players: {response.players.online}/{response.players.max} {response.players.names}")
 
 
 if __name__ == "__main__":
-    cli()
+    cli()  # type: ignore[call-arg]
